@@ -1,4 +1,4 @@
-# Copyright 2021 The TensorFlow Recommenders Authors.
+# Copyright 2022 The TensorFlow Recommenders Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,7 +13,7 @@
 # limitations under the License.
 
 # pylint: disable=g-long-lambda
-"""Tests for tensorflow_recommenders.experimental.models.ranking_model."""
+"""Tests for Ranking."""
 
 import itertools
 import math
@@ -66,6 +66,8 @@ def _generate_synthetic_data(num_dense: int,
                              generate_weights: bool = False) -> tf.data.Dataset:
   dense_tensor = tf.random.uniform(
       shape=(dataset_size, num_dense), maxval=1.0, dtype=tf.float32)
+  # The mean is in [0, 1] interval.
+  dense_tensor_mean = tf.math.reduce_mean(dense_tensor, axis=1)
 
   sparse_tensors = []
   for size in vocab_sizes:
@@ -77,9 +79,6 @@ def _generate_synthetic_data(num_dense: int,
       str(i): sparse_tensors[i] for i in range(len(sparse_tensors))
   }
 
-  # The mean is in [0, 1] interval.
-  dense_tensor_mean = tf.math.reduce_mean(dense_tensor, axis=1)
-
   sparse_tensors = tf.stack(sparse_tensors, axis=-1)
   sparse_tensors_mean = tf.math.reduce_sum(sparse_tensors, axis=1)
   # The mean is in [0, 1] interval.
@@ -143,7 +142,7 @@ class RankingTest(tf.test.TestCase, parameterized.TestCase):
 
     embedding_feature_config = _get_tpu_embedding_feature_config(
         vocab_sizes=vocabulary_sizes, embedding_dim=16)
-    optimizer = tf.keras.optimizers.Adam()
+    optimizer = tf.keras.optimizers.legacy.Adam()
 
     model = tfrs.experimental.models.Ranking(
         embedding_layer=tfrs.experimental.layers.embedding.PartialTPUEmbedding(
