@@ -134,8 +134,11 @@ def _clone_and_prepare_features(feature_config):
             dim=config.table.dim,
             initializer=config.table.initializer,
             optimizer=config.table.optimizer,
+            quantization_config=config.table.quantization_config,
             combiner=config.table.combiner,
-            name=config.table.name))
+            name=config.table.name,
+        ),
+    )
 
     output_objects.append(
         tf.tpu.experimental.embedding.FeatureConfig(
@@ -189,6 +192,7 @@ def _update_table_configs(feature_config, table_config_map):
             table=table_config_dict[config.table],
             max_sequence_length=config.max_sequence_length,
             validate_weights_and_indices=config.validate_weights_and_indices,
+            output_shape=config.output_shape,
             name=config.name))
 
   return tf.nest.pack_sequence_as(feature_config, output_objects)
