@@ -1,4 +1,4 @@
-# Copyright 2021 The TensorFlow Recommenders Authors.
+# Copyright 2022 The TensorFlow Recommenders Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -154,4 +154,5 @@ class SamplingProbablityCorrection(tf.keras.layers.Layer):
                candidate_sampling_probability: tf.Tensor) -> tf.Tensor:
     """Corrects the input logits to account for candidate sampling probability."""
 
-    return logits - tf.math.log(candidate_sampling_probability)
+    return logits - tf.math.log(
+        tf.clip_by_value(candidate_sampling_probability, 1e-6, 1.))
