@@ -15,6 +15,8 @@
 # lint-as: python3
 """Factorized retrieval top K metrics."""
 
+import abc
+
 from typing import List, Optional, Sequence, Union
 
 import tensorflow as tf
@@ -22,7 +24,32 @@ import tensorflow as tf
 from tensorflow_recommenders import layers
 
 
-class FactorizedTopK(tf.keras.layers.Layer):
+class Factorized(tf.keras.layers.Layer, abc.ABC):
+  """Computes metrics across top K candidates surfaced by a retrieval model."""
+
+  @abc.abstractmethod
+  def update_state(
+      self,
+      query_embeddings: tf.Tensor,
+      true_candidate_embeddings: tf.Tensor,
+      true_candidate_ids: Optional[tf.Tensor] = None
+  ) -> tf.Operation:
+
+    raise NotImplementedError()
+
+  def reset_states(self) -> None:
+    """Resets the metrics."""
+
+    for metric in self.metrics:
+      metric.reset_states()
+
+  def result(self) -> List[tf.Tensor]:
+    """Returns a list of metric results."""
+
+    return [metric.result() for metric in self.metrics]
+
+
+class FactorizedTopK(Factorized):
   """Computes metrics for across top K candidates surfaced by a retrieval model.
 
   The default metric is top K categorical accuracy: how often the true candidate
@@ -65,7 +92,8 @@ class FactorizedTopK(tf.keras.layers.Layer):
       self,
       query_embeddings: tf.Tensor,
       true_candidate_embeddings: tf.Tensor,
-      true_candidate_ids: Optional[tf.Tensor] = None
+      true_candidate_ids: Optional[tf.Tensor] = None,
+      sample_weight: Optional[tf.Tensor] = None,
   ) -> tf.Operation:
     """Updates the metrics.
 
@@ -88,6 +116,7 @@ class FactorizedTopK(tf.keras.layers.Layer):
         by multiplying the candidate and embedding vector. For example, scores
         returned by ScaNN are quantized, and cannot be compared to
         full-precision scores.
+      sample_weight: Optional weighting of each example. Defaults to 1.
 
     Returns:
       Update op. Only used in graph mode.
@@ -148,7 +177,7 @@ class FactorizedTopK(tf.keras.layers.Layer):
             tf.reduce_sum(ids_match[:, :k], axis=1, keepdims=True),
             0.0, 1.0
         )
-        update_ops.append(metric.update_state(match_found))
+        update_ops.append(metric.update_state(match_found, sample_weight))
     else:
       # Score-based evaluation.
       y_pred = tf.concat([positive_scores, top_k_predictions], axis=1)
@@ -160,17 +189,7 @@ class FactorizedTopK(tf.keras.layers.Layer):
             predictions=y_pred,
             k=k
         )
-        update_ops.append(metric.update_state(top_k_accuracy))
+        update_ops.append(metric.update_state(top_k_accuracy, sample_weight))
 
     return tf.group(update_ops)
-
-  def reset_states(self) -> None:
-    """Resets the metrics."""
-
-    for metric in self.metrics:
-      metric.reset_states()
-
-  def result(self) -> List[tf.Tensor]:
-    """Returns a list of metric results."""
-
-    return [metric.result() for metric in self.metrics]
+  
\ No newline at end of file
